@@ -6,6 +6,7 @@ except ImportError:
 from bs4 import BeautifulSoup
 from devpi_common.archive import Archive
 from devpi_common.types import cached_property
+from devpi_common.validation import normalize_name
 from devpi_server.log import threadlog
 import json
 
@@ -13,16 +14,28 @@ import json
 def get_unpack_path(stage, name, version):
     # XXX this should rather be in some devpi-web managed directory area
     return stage.keyfs.basedir.join(stage.user.name, stage.index,
-                                    name, version, "+doc")
+                                    normalize_name(name), version, "+doc")
 
 
 def unpack_docs(stage, name, version, entry):
     # unpack, maybe a bit uncarefully but in principle
     # we are not loosing the original zip file anyway
     unpack_path = get_unpack_path(stage, name, version)
+    hash_path = unpack_path.join('.hash')
+    if hash_path.exists():
+        with hash_path.open() as f:
+            if f.read().strip() == entry.hash_spec:
+                threadlog.debug(
+                    "%s: returning existing docs with hash_spec %s",
+                    stage.name, entry.hash_spec)
+                return unpack_path
+    if unpack_path.exists():
+        unpack_path.remove()
     with entry.file_open_read() as f:
         with Archive(f) as archive:
             archive.extract(unpack_path)
+    with hash_path.open('w') as f:
+        f.write(entry.hash_spec)
     threadlog.debug("%s: unpacked %s-%s docs to %s",
                     stage.name, name, version, unpack_path)
     return unpack_path
@@ -35,11 +48,15 @@ class Docs(DictMixin):
         self.version = version
         self.unpack_path = get_unpack_path(stage, name, version)
 
+    def exists(self):
+        return self.unpack_path.exists()
+
     @cached_property
     def _entries(self):
-        if not self.unpack_path.exists():
+        if not self.exists():
             # this happens on import, when the metadata is registered, but the docs
             # aren't uploaded yet
+            threadlog.warn("Tried to access %s, but it doesn't exist.", self.unpack_path)
             return {}
         html = []
         fjson = []
@@ -78,11 +95,11 @@ class Docs(DictMixin):
         if entry.basename.endswith('.fjson'):
             info = json.loads(entry.read())
             return dict(
-                title=BeautifulSoup(info.get('title', '')).text,
-                text=BeautifulSoup(info.get('body', '')).text,
+                title=BeautifulSoup(info.get('title', ''), "html.parser").text,
+                text=BeautifulSoup(info.get('body', ''), "html.parser").text,
                 path=info.get('current_page_name', name))
         else:
-            soup = BeautifulSoup(entry.read())
+            soup = BeautifulSoup(entry.read(mode='rb'), "html.parser")
             body = soup.find('body')
             if body is None:
                 return
@@ -95,12 +112,3 @@ class Docs(DictMixin):
                 title=title,
                 text=body.text,
                 path=name)
-
-
-def iter_doc_contents(stage, name, version):
-    docs = Docs(stage, name, version)
-    for entry in docs:
-        result = docs[entry]
-        if result is None:
-            continue
-        yield result
