@@ -1,12 +1,14 @@
 from __future__ import unicode_literals
 from chameleon.config import AUTO_RELOAD
 from devpi_common.metadata import get_latest_version
+from devpi_web import hookspecs
 from devpi_web.description import render_description
 from devpi_web.doczip import unpack_docs
 from devpi_web.indexing import iter_projects, preprocess_project
 from devpi_web.whoosh_index import Index
 from devpi_server.log import threadlog
 from pkg_resources import resource_filename
+from pluggy import PluginManager
 from pyramid.renderers import get_renderer
 from pyramid_chameleon.renderer import ChameleonRendererLookup
 import os
@@ -38,10 +40,10 @@ def navigation_version(context):
     version = context.version
     if version == 'latest':
         stage = context.model.getstage(context.username, context.index)
-        version = stage.get_latest_version(context.name)
+        version = stage.get_latest_version(context.project)
     elif version == 'stable':
         stage = context.model.getstage(context.username, context.index)
-        version = stage.get_latest_version(context.name, stable=True)
+        version = stage.get_latest_version(context.project, stable=True)
     return version
 
 
@@ -62,11 +64,11 @@ def navigation_info(request):
             title="%s/%s" % (user, index)))
     else:
         return result
-    if 'name' in context.matchdict:
-        name = context.name
+    if 'project' in context.matchdict:
+        name = context.project
         path.append(dict(
             url=request.route_url(
-                "/{user}/{index}/{name}", user=user, index=index, name=name),
+                "/{user}/{index}/{project}", user=user, index=index, project=name),
             title=name))
     else:
         return result
@@ -74,14 +76,34 @@ def navigation_info(request):
         version = navigation_version(context)
         path.append(dict(
             url=request.route_url(
-                "/{user}/{index}/{name}/{version}",
-                user=user, index=index, name=name, version=version),
+                "/{user}/{index}/{project}/{version}",
+                user=user, index=index, project=name, version=version),
             title=version))
     else:
         return result
     return result
 
 
+def status_info(request):
+    msgs = []
+    pm = request.registry['devpiweb-pluginmanager']
+    for result in pm.hook.devpiweb_get_status_info(request=request):
+        for msg in result:
+            msgs.append(msg)
+    states = set(x['status'] for x in msgs)
+    if 'fatal' in states:
+        status = 'fatal'
+        short_msg = 'fatal'
+    elif 'warn' in states:
+        status = 'warn'
+        short_msg = 'degraded'
+    else:
+        status = 'ok'
+        short_msg = 'ok'
+    url = request.route_url('/+status')
+    return dict(status=status, short_msg=short_msg, msgs=msgs, url=url)
+
+
 def query_docs_html(request):
     search_index = request.registry['search_index']
     return search_index.get_query_parser_html_help()
@@ -101,14 +123,24 @@ class ThemeChameleonRendererLookup(ChameleonRendererLookup):
         return ChameleonRendererLookup.__call__(self, info)
 
 
+def get_pluginmanager(load_entry_points=True):
+    pm = PluginManager("devpiweb", implprefix="devpiweb_")
+    pm.add_hookspecs(hookspecs)
+    if load_entry_points:
+        pm.load_setuptools_entrypoints("devpi_web")
+    pm.check_pending()
+    return pm
+
+
 def includeme(config):
+    from devpi_web import __version__
     from pyramid_chameleon.interfaces import IChameleonLookup
     from pyramid_chameleon.zpt import ZPTTemplateRenderer
     config.include('pyramid_chameleon')
     # we overwrite the template lookup to allow theming
     lookup = ThemeChameleonRendererLookup(ZPTTemplateRenderer, config.registry)
     config.registry.registerUtility(lookup, IChameleonLookup, name='.pt')
-    config.add_static_view('+static', 'static')
+    config.add_static_view('+static-%s' % __version__, 'static')
     theme_path = config.registry['theme_path']
     if theme_path:
         # if a theme is used, we set the path on the lookup instance
@@ -117,26 +149,28 @@ def includeme(config):
         # method 'theme_static_url' on the request
         static_path = os.path.join(theme_path, 'static')
         if os.path.exists(static_path):
-            config.add_static_view('+theme-static', static_path)
+            config.add_static_view('+theme-static-%s' % __version__, static_path)
             config.add_request_method(theme_static_url)
     config.add_route('root', '/', accept='text/html')
     config.add_route('search', '/+search', accept='text/html')
     config.add_route('search_help', '/+searchhelp', accept='text/html')
     config.add_route(
         "docroot",
-        "/{user}/{index}/{name}/{version}/+doc/{relpath:.*}")
+        "/{user}/{index}/{project}/{version}/+doc/{relpath:.*}")
     config.add_route(
         "docviewroot",
-        "/{user}/{index}/{name}/{version}/+d/{relpath:.*}")
+        "/{user}/{index}/{project}/{version}/+d/{relpath:.*}")
     config.add_route(
         "toxresults",
-        "/{user}/{index}/{name}/{version}/+toxresults/{basename}")
+        "/{user}/{index}/{project}/{version}/+toxresults/{basename}")
     config.add_route(
         "toxresult",
-        "/{user}/{index}/{name}/{version}/+toxresults/{basename}/{toxresult}")
+        "/{user}/{index}/{project}/{version}/+toxresults/{basename}/{toxresult}")
     config.add_request_method(macros, reify=True)
     config.add_request_method(navigation_info, reify=True)
+    config.add_request_method(status_info, reify=True)
     config.add_request_method(query_docs_html, reify=True)
+    config.registry['devpiweb-pluginmanager'] = get_pluginmanager()
     config.scan()
 
 
@@ -197,17 +231,23 @@ def devpiserver_add_parser_options(parser):
              "without a full devpi-server import/export.")
 
 
-def devpiserver_pypi_initial(stage, name2serials):
+def devpiserver_mirror_initialnames(stage, projectnames):
     xom = stage.xom
     ix = get_indexer(xom.config)
     ix.delete_index()
     indexer = get_indexer(xom.config)
-    # directly use name2serials?
+    # directly use projectnames?
     indexer.update_projects(iter_projects(xom), clear=True)
     threadlog.info("finished initial indexing op")
 
 
-def devpiserver_run_commands(xom):
+def devpiserver_stage_created(stage):
+    if stage.ixconfig["type"] == "mirror":
+        threadlog.info("triggering load of initial projectnames for %s", stage.name)
+        stage.list_projects_perstage()
+
+
+def devpiserver_cmdline_run(xom):
     ix = get_indexer(xom.config)
     if xom.config.args.recreate_search_index:
         ix.delete_index()
@@ -233,31 +273,31 @@ def index_project(stage, name):
     ix.update_projects([preprocess_project(stage, name)])
 
 
-def devpiserver_on_upload(stage, projectname, version, link):
+def devpiserver_on_upload(stage, project, version, link):
     if not link.entry.file_exists():
         # on replication or import we might be at a lower than
         # current revision and the file might have been deleted already
-        threadlog.debug("igoring lost upload: %s", link)
+        threadlog.debug("ignoring lost upload: %s", link)
     elif link.rel == "doczip":
-        unpack_docs(stage, projectname, version, link.entry)
-        index_project(stage, projectname)
+        unpack_docs(stage, project, version, link.entry)
+        index_project(stage, project)
 
 
-def devpiserver_on_changed_versiondata(stage, projectname, version, metadata):
+def devpiserver_on_changed_versiondata(stage, project, version, metadata):
     if stage is None:
         # TODO we don't have enough info to delete the project
         return
     if not metadata:
-        if stage.get_projectname(projectname) is None:
-            delete_project(stage, projectname)
+        if not stage.has_project_perstage(project):
+            delete_project(stage, project)
             return
-        versions = stage.list_versions(projectname)
+        versions = stage.list_versions(project)
         if versions:
             version = get_latest_version(versions)
             if version:
                 threadlog.debug("A version of %s was deleted, using latest version %s for indexing" % (
-                    projectname, version))
-                metadata = stage.get_versiondata(projectname, version)
+                    project, version))
+                metadata = stage.get_versiondata(project, version)
     if metadata:
         render_description(stage, metadata)
         index_project(stage, metadata['name'])
